/*
* Copyright © 2018 - 2019 Virtru Corporation
*
* SPDX - License - Identifier: MIT
*
*/
//
//  Virtru TDF3 SDK
//
#include <stdio.h>
#include <virtru_constants_c.h>
#include <virtru_client_c.h>
#include <virtru_encrypt_file_params_c.h>
#include <virtru_encrypt_string_params_c.h>
#include <virtru_policy_c.h>
#include <stdlib.h>
#include <string.h>

/* User credentials */
const char owner[] = "11111111-1111-1111-1111-111111111111"; /* Put owner email here */
const char appId[] = "your-user-name@your-domain.com"; /* Put owner appId here */

int fileAPISample() {

    const char sampleFile[] = "sample.txt";
    const char sampleTdfFile[] = "sample.txt.tdf";
    const char sampleTdfOutFile[] = "sample-result.txt";

    /* Create a sample data */
    FILE *fp;
    if((fp = fopen(sampleFile, "w")) == NULL) {
        printf ("Cannot open file.\n");
        return EXIT_FAILURE;
    }

    fprintf(fp, "Hello Word!!");
    fclose(fp);

    VClientPtr clientPtr = VClientCreateWithAppId(owner, appId);
    if (clientPtr == NULL) {
        printf("Fail to create virtru client.\n");
        return EXIT_FAILURE;
    }
    VSTATUS statusCode;

    statusCode = VEnableConsoleLogging(clientPtr, VLogLevelInfo);
    if (statusCode != VSTATUS_SUCCESS) {
        printf("Fail to enable console logging.\n");
        VClientDestroy(clientPtr);
        return EXIT_FAILURE;
    }

    statusCode = VSetProtocol(clientPtr, VProtocolZip);
    if (statusCode != VSTATUS_SUCCESS) {
        printf("Fail to set tdf protocol on virtru client object.\n");

        VClientDestroy(clientPtr);
        return EXIT_FAILURE;
    }

    VEncryptFileParamsPtr vEncryptFileParamsPtr = VEncryptFileParamsCreate2(sampleFile, sampleTdfFile);
    if (vEncryptFileParamsPtr == NULL) {
        printf("Fail to create Encrypt file param object.\n");

        VClientDestroy(clientPtr);
        return EXIT_FAILURE;
    }

    statusCode = VEncryptFileParamsSetDisplayName(vEncryptFileParamsPtr, "Encrypt with C Interface");
    if (statusCode != VSTATUS_SUCCESS) {
        printf("Fail to set display name on encrypt file param object.\n");

        VClientDestroy(clientPtr);
        VEncryptFileParamsDestroy(vEncryptFileParamsPtr);
        return EXIT_FAILURE;
    }

    /* Create a policy to expire the tdf in 5 mins. */
    VPolicyPtr policyPtr = VPolicyCreate();
    if (policyPtr == NULL) {
        printf("Fail to create policy object.\n");

        VEncryptFileParamsDestroy(vEncryptFileParamsPtr);
        VClientDestroy(clientPtr);
        return EXIT_FAILURE;
    }

    /* Set the policy to expire the tdf in 5 mins. */
    statusCode = VExpireInMins(policyPtr, 5);
    if (statusCode != VSTATUS_SUCCESS) {
        printf("Fail to set policy to expire the tdf in 5 mins.\n");

        VClientDestroy(clientPtr);
        VPolicyDestroy(policyPtr);
        VEncryptFileParamsDestroy(vEncryptFileParamsPtr);
        return EXIT_FAILURE;
    }

    /* Set the policy on encrypt file params object. */
    statusCode = VEncryptFileParamsSetPolicy(vEncryptFileParamsPtr, policyPtr);
    if (statusCode != VSTATUS_SUCCESS) {
        printf("Fail to set policy on encrypt file param object.\n");

        VClientDestroy(clientPtr);
        VPolicyDestroy(policyPtr);
        VEncryptFileParamsDestroy(vEncryptFileParamsPtr);
        return EXIT_FAILURE;
    }

    /* Encrypt file */
    char* policyId = NULL;
    statusCode = VClientEncryptFile(clientPtr, vEncryptFileParamsPtr, &policyId);
    if (statusCode != VSTATUS_SUCCESS) {
        printf("Fail to encrypt file.\n");

        VClientDestroy(clientPtr);
        VPolicyDestroy(policyPtr);
        VEncryptFileParamsDestroy(vEncryptFileParamsPtr);
        return EXIT_FAILURE;
    }

    VPolicyDestroy(policyPtr);
    VEncryptFileParamsDestroy(vEncryptFileParamsPtr);

    printf("PolicyId:%s\n", policyId);
    free(policyId);

    /* Decrypt file */
    statusCode = VClientDecryptFile(clientPtr, sampleTdfFile, sampleTdfOutFile);
    if (statusCode != VSTATUS_SUCCESS) {
        printf("Fail to decrypt file.\n");

        VClientDestroy(clientPtr);
        return EXIT_FAILURE;
    }

    printf("Encrypt/Decrypt successfully using C interface!!\n");

    VClientDestroy(clientPtr);
    return EXIT_SUCCESS;
}

int stringAPISample() {
    const unsigned char plainText[] = "Hello world!!";

    VClientPtr clientPtr = VClientCreateWithAppId(owner, appId);
    if (clientPtr == NULL) {
        printf("Fail to create virtru client.\n");
        return EXIT_FAILURE;
    }
    VSTATUS statusCode;

    statusCode = VEnableConsoleLogging(clientPtr, VLogLevelDebug);
    if (statusCode != VSTATUS_SUCCESS) {
        printf("Fail to enable console logging.\n");
        VClientDestroy(clientPtr);
        return EXIT_FAILURE;
    }

    statusCode = VSetProtocol(clientPtr, VProtocolHtml);
    if (statusCode != VSTATUS_SUCCESS) {
        printf("Fail to set tdf protocol on virtru client object.\n");

        VClientDestroy(clientPtr);
        return EXIT_FAILURE;
    }

    VEncryptStringParamsPtr vEncryptStringParamsPtr = VEncryptStringParamsCreate(plainText, strlen((char*)plainText));
    if (vEncryptStringParamsPtr == NULL) {
        printf("Fail to create Encrypt string param object.\n");

        VClientDestroy(clientPtr);
        return EXIT_FAILURE;
    }

    /* Encrypt string */
    char* policyId = NULL;
    unsigned char* tdfData = NULL;
    unsigned int tdfDataLength;
    statusCode = VClientEncryptString(clientPtr, vEncryptStringParamsPtr, &policyId, &tdfData, &tdfDataLength);
    if (statusCode != VSTATUS_SUCCESS) {
        printf("Fail to encrypt string.\n");

        VClientDestroy(clientPtr);
        VEncryptStringParamsDestroy(vEncryptStringParamsPtr);
        return EXIT_FAILURE;
    }

    VEncryptStringParamsDestroy(vEncryptStringParamsPtr);

    printf("PolicyId:%s\n", policyId);
    free(policyId);

    unsigned char* plainData = NULL;
    unsigned int plainDataLength;

    /* Decrypt string */
    statusCode = VClientDecryptString(clientPtr, tdfData, tdfDataLength, &plainData, &plainDataLength);
    if (statusCode != VSTATUS_SUCCESS) {
        printf("Fail to decrypt tdf data.\n");

        VClientDestroy(clientPtr);
        return EXIT_FAILURE;
    }

    if (strncmp((const char*)plainText, (const char*)plainData, plainDataLength) != 0) {
        printf("Encrypt/Decrypt using string API failed!!\n");

        VClientDestroy(clientPtr);
        return EXIT_FAILURE;
    }

    printf("Encrypt/Decrypt successfully using using string API !!\n");
    free(plainData);
    free(tdfData);

    VClientDestroy(clientPtr);
    return EXIT_SUCCESS;
}

int main() {

    int ret = fileAPISample();
    if (ret != EXIT_SUCCESS) {
        printf("File API sample tests failed !!\n");
    }

    ret = stringAPISample();
    if (ret != EXIT_SUCCESS) {
        printf("String API sample tests failed !!\n");
    }

    return EXIT_SUCCESS;
}